/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

// DEPRECATED: This file exists for shim purposes only,
// see MenuBoxObject.webidl

[uuid(689ebf3d-0184-450a-9bfa-5a26be0e7a8c)]
interface nsIMenuBoxObject : nsISupports
{
};
